﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Hive.Slave.Communication.SlaveService;
using HeuristicLab.Hive.Contracts;
using System.ServiceModel;
using HeuristicLab.Hive.Slave.Communication.Properties;
using HeuristicLab.Tracing;

namespace HeuristicLab.Hive.Slave.Communication {
  internal static class ServiceLocator {
    /// <summary>
    /// Cache of SlaveFacadeClients, to check if all get disposed
    /// </summary>
    private static List<ISlaveFacade> clientCache = new List<ISlaveFacade>();

    internal static ISlaveFacade CreateSlaveFacade(string hostAddress) {
      SlaveFacadeClient client = new SlaveFacadeClient("SlaveHttpEndpoint");
      WcfSettings.SetEndpointAddress(client.Endpoint, hostAddress);
      SetCredentials(client);
      client.Open();
      clientCache.Add(client);
      Logger.Debug("Created SlaveFacadeClients. Currently existing SlaveFacadeClients: " + clientCache.Count);
      return client;
    }

    internal static ISlaveFacade CreateStreamedSlaveFacade(string hostAddress) {
      SlaveFacadeClient client = new SlaveFacadeClient("SlaveTcpStreamedEndpoint");
      WcfSettings.SetEndpointAddress(client.Endpoint, hostAddress);
      SetCredentials(client);
      client.Open();
      clientCache.Add(client);
      Logger.Debug("Created SlaveFacadeClients. Currently existing SlaveFacadeClients: " + clientCache.Count);
      return client;
    }

    private static void SetCredentials(SlaveFacadeClient client) {
      client.ClientCredentials.UserName.UserName = Settings.Default.HiveUsername;
      client.ClientCredentials.UserName.Password = Settings.Default.HivePassword;
    }

    public static void DisposeSlaveClient(ISlaveFacade slaveFacade) {
      WcfSettings.DisposeWcfClient((ICommunicationObject)slaveFacade);
      clientCache.Remove(slaveFacade);
      Logger.Debug("Disposing SlaveFacadeClients. Currently existing SlaveFacadeClients: " + clientCache.Count);
    }
  }
}
